import * as React from 'react';
export interface LCMarqueeProps {
    text: 'string';
    /**
     * Whether to play or pause the marquee
     * Type: boolean
     * Default: true
     */
    play?: boolean;
    /**
     * Whether to pause the marquee when hovered
     * Type: boolean
     * Default: false
     */
    pauseOnHover?: boolean;
    /**
     * Whether to pause the marquee when clicked
     * Type: boolean
     * Default: false
     */
    pauseOnClick?: boolean;
    /**
     * The direction the marquee is sliding
     * Type: "left" or "right"
     * Default: "left"
     */
    direction?: 'left' | 'right';
    /**
     * Speed calculated as pixels/second
     * Type: number
     * Default: 20
     */
    speed?: number;
    /**
     * Duration to delay the animation after render, in seconds
     * Type: number
     * Default: 0
     */
    delay?: number;
    /**
     * The number of times the marquee should loop, 0 is equivalent to infinite
     * Type: number
     * Default: 0
     */
    loop?: number;
    /**
     * Whether to show the gradient or not
     * Type: boolean
     * Default: true
     */
    gradient?: boolean;
    /**
     * The rgb color of the gradient as an array of length 3
     * Type: Array<number> of length 3
     * Default: [255, 255, 255]
     */
    gradientColor?: [number, number, number];
    /**
     * The width of the gradient on either side
     * Type: string
     * Default: 200
     */
    gradientWidth?: number | string;
}
declare const LCMarquee: React.FC<LCMarqueeProps>;
export default LCMarquee;
